package get

import (
	"slices"
	"strings"

	"github.com/k0kubun/pp/v3"
	"github.com/spf13/cobra"

	"gitea.libretechconsulting.com/50W/eia-api-go/cmd/eia-client/internal/util"
	"gitea.libretechconsulting.com/50W/eia-api-go/pkg/eia"
)

var (
	filteredSuffixes = []string{
		"Facet",
		"FacetId",
		"Body",
	}
	filteredPrefixes = []string{
		"EIAA",
	}
)

var GetRouteCmd = &cobra.Command{
	Use:               "route",
	Aliases:           []string{"r"},
	Short:             "Describe a route",
	Args:              cobra.ExactArgs(1),
	ValidArgsFunction: completeRouteRoutes,
	Run:               RunGetRouteCmd,
}

func RunGetRouteCmd(cmd *cobra.Command, args []string) {
	logger := util.Logger(cmd)
	logger.Info().Str("route", args[0]).Msg("getting route description")

	route, err := util.GetRoute(cmd, args[0])
	if err != nil {
		logger.Fatal().Err(err).Send()
	}

	pp.Println(route)
}

func completeRouteRoutes(cmd *cobra.Command, args []string, toComplete string) (
	[]string, cobra.ShellCompDirective,
) {
	routes := eia.GetRoutes("Data")
	routeRoutes := make([]string, 0, len(routes))
	for _, r := range routes {
		var rName string
		for _, suffix := range filteredSuffixes {
			if strings.HasSuffix(r, suffix) {
				goto next
			}
		}
		for _, prefix := range filteredPrefixes {
			if strings.HasPrefix(r, prefix) {
				goto next
			}
		}
		rName, _ = strings.CutSuffix(r, "Data")
		routeRoutes = append(routeRoutes, rName)
	next:
	}

	filteredRoutes := make([]string, 0, len(routeRoutes))
	if toComplete != "" {
		for _, route := range routeRoutes {
			if strings.HasPrefix(route, toComplete) {
				filteredRoutes = append(filteredRoutes, route)
			}
		}
	} else {
		filteredRoutes = routeRoutes
	}
	return slices.Clip(filteredRoutes), cobra.ShellCompDirectiveNoFileComp
}
