package list

import (
	"strings"

	"github.com/k0kubun/pp/v3"
	"github.com/spf13/cobra"

	"gitea.libretechconsulting.com/50W/eia-api-go/cmd/eia-client/internal/util"
	"gitea.libretechconsulting.com/50W/eia-api-go/pkg/eia"
)

var listRoutesCmd = &cobra.Command{
	Use:     "routes",
	Aliases: []string{"methods", "r"},
	Short:   "List routes, optionally by type (default is Data)",
	Args:    cobra.MaximumNArgs(2),
	ValidArgs: []string{
		"Data",
		"Facet",
	},
	Run: RunListRoutesCmd,
}

const (
	allMethodsFlag  = "all"
	filtersFlag     = "filters"
	routePrefixFlag = "routePrefix"
)

func RunListRoutesCmd(cmd *cobra.Command, args []string) {
	logger := util.Logger(cmd)

	// Command flags
	allRoutes, _ := cmd.Flags().GetBool(allMethodsFlag)
	routePrefix, _ := cmd.Flags().GetString(routePrefixFlag)
	filters, _ := cmd.Flags().GetStringSlice(filtersFlag)

	if allRoutes && len(args) > 0 {
		logger.Fatal().Msg("can't specify all methods while also filtering by type")
	}

	filter := make([]string, 0)
	if !allRoutes && len(args) == 1 {
		filter = append(filter, args[0])
	} else if !allRoutes {
		filter = append(filter, "Data")
	}

	routes := eia.GetRoutes(filter...)
	filteredRoutes := make([]string, 0, len(routes))

	// Apply filters for routePrefix, and optional filters
	for _, route := range routes {
		if !allRoutes && routePrefix != "" {
			if !strings.HasPrefix(strings.ToLower(route), strings.ToLower(routePrefix)) {
				continue
			}
		}

		if len(filters) > 0 {
			for _, f := range filters {
				if strings.Contains(strings.ToLower(route), strings.ToLower(f)) {
					filteredRoutes = append(filteredRoutes, route)
				}
			}
		} else {
			filteredRoutes = append(filteredRoutes, route)
		}
	}

	pp.Println(filteredRoutes)
}

func init() {
	listRoutesCmd.PersistentFlags().BoolP(allMethodsFlag, "a", false, "List all methods, no filtering")
	listRoutesCmd.PersistentFlags().StringSliceP(filtersFlag, "f", []string{}, "Optional filters, case insensitive")
	listRoutesCmd.PersistentFlags().StringP(routePrefixFlag, "p", "Get", "Prefix for routes, ignore with -a flag")
}
